@@ -3,7 +3,7 @@
    Network input dispatcher... */
 
 /*
- * Copyright (c) 2004-2005 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2007 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1995-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -34,7 +34,7 @@
 
 #ifndef lint
 static char copyright[] =
-"$Id: discover.c,v 1.42.2.17 2005/03/03 16:55:22 dhankins Exp $ Copyright (c) 2004-2005 Internet Systems Consortium.  All rights reserved.\n";
+"$Id: discover.c,v 1.42.2.23 2007/05/01 20:42:55 each Exp $ Copyright (c) 2004-2006 Internet Systems Consortium.  All rights reserved.\n";
 #endif /* not lint */
 
 #include "dhcpd.h"
@@ -463,6 +463,18 @@ void discover_interfaces (state)
 #ifdef HAVE_ARPHRD_ROSE
 		      case ARPHRD_ROSE:
 #endif
+#ifdef HAVE_ARPHRD_IRDA
+		     case ARPHRD_IRDA:
+			/* ignore infrared interfaces. */
+#endif
+#ifdef HAVE_ARPHRD_SIT
+		     case ARPHRD_SIT:
+			/* ignore IPv6-in-IPv4 interfaces. */
+#endif
+#ifdef HAVE_ARPHRD_IEEE1394
+		     case ARPHRD_IEEE1394:
+			/* ignore IEEE1394 interfaces. */
+#endif
 #ifdef HAVE_ARPHRD_LOOPBACK
 		      case ARPHRD_LOOPBACK:
 			/* ignore loopback interface */
@@ -926,7 +938,7 @@ isc_result_t dhcp_interface_stuff_values (omapi_object_t *c,
 	status = omapi_connection_put_name (c, "state");
 	if (status != ISC_R_SUCCESS)
 		return status;
-	if (interface -> flags && INTERFACE_REQUESTED)
+	if ((interface->flags & INTERFACE_REQUESTED) != 0)
 	    status = omapi_connection_put_string (c, "up");
 	else
 	    status = omapi_connection_put_string (c, "down");
