@@ -3,7 +3,7 @@
    Functions for maintaining handles on objects. */
 
 /*
- * Copyright (c) 2004 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2006 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1999-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -32,6 +32,11 @@
  * ``http://www.nominum.com''.
  */
 
+#ifndef lint
+static char ocopyright[] =
+"$Id: handle.c,v 1.6.2.4 2006/02/22 22:43:27 dhankins Exp $ Copyright 2004-2006 Internet Systems Consortium.";
+#endif
+
 #include <omapip/omapip_p.h>
 
 /* The handle table is a hierarchical tree designed for quick mapping
